//
//  OAuthToken.swift
//  Busca Pios
//
//  Created by Santiago Pavón on 17/12/14.
//  Copyright (c) 2014 UPM. All rights reserved.
//

import Foundation

// Clave y password de acceso de la aplicacion a Twitter.
//
// Hay que dar de alta una nueva aplicacion en twitter e introducir en las
// constante de abajo los valores CONSUMER_KEY y CONSUMER_SECRET asignados
// a la aplicacion.
//
// Para obtener estos valores hay que dar de alta una aplicacion en Twitter.
// - Conectarse a https://dev.twitter.com
// - Pulsar "Sign in" para entrar en su cuenta de usuario.
//    o Meter el Username y el Password.
// - Desplegar el menu asociado a la foto del perfil y seleccionar "My Applications".
// - Pulsar el boton "Create a new application".
//    o Rellenar los campos del formulario: Name, Description y Web Site.
//    o Leerse y aceptar las "Rules of the Road".
//    o Escribir el Captcha
//    o Pulsar el boton "Create your Twitter application".
// - Y nos sale una pagina con la Consumer Key y la Consumer Secret que debe
//   utilizar la aplicacion que acabamos de crear.
//
private let  CONSUMER_KEY    = "METE AQUI TU CONSUMER KEY"
private let  CONSUMER_SECRET = "METE AQUI TU CONSUMER SECRET"


/**
*  Datos sobre la peticion que hay que realizar para obtener el token de 
*  acceso qnecesario para acceder al API de busquedas.
*
*  La peticion HTTP:
*     - URL = https://api.twitter.com/oauth2/token
*     - metodo = POST
*     - cabecera = Authorization: Basic $Base64BearerToken
*     - cabecera = Content-Type: application/x-www-form-urlencoded;charset=UTF-8
*     - body = grant_type=client_credentials
*
*  La respuesta HTTP:
*     - Si hay exito:
*           { "access_token":"el_valor_del_token","token_type":"bearer"}
*     - Si hay errores:
*           {"errors":[{"label":"authenticity_token_error","code":99,
*                       "message":"Unable to verify your credentials"}]}
*           {"errors":[{"message":"Bad Authentication data","code":215}]}
*           u otros ...
*/


private let OAUTH_URL = "https://api.twitter.com/oauth2/token"


/**
*  Clase auxiliar con lo necesario para obtener el token de acceso.
*
*  Sobre los threads:
*   - Esta clase no es thread-safe.
*   - El closure completion del metodo getAccessToken puede ejecutarse 
*     en cualquier thread, es decir, distinto del thread que llamo al
*     metodo de obtener el access token.
*/
class OAuthToken {
    
    // Propiedades estaticas
    struct Static {
      
        // Token de acceso devuelto por el servicio Twitter para que podamos 
        // acceder a su API.
        // Lo almacenamos en una variable estatica para conservar su valor 
        // durante toda la ejecucion de la aplicacion. Asi solo debe obtenerse 
        // este valor una sola vez.
        static var access_token: String?
    }
    

    // Obtener el token de acceso y despues ejecutar el closure pasandole 
    // el token o nil.
    class func getAccessToken(completion: (token: String?) -> Void) {
        
        println("Obteniendo el token de acceso.")

        if Static.access_token != nil { // Ya lo tengo
            completion(token: Static.access_token)
            return
        }
        
        // Peticion HTTP:
        let url = NSURL(string: OAUTH_URL)!
        let req = NSMutableURLRequest(URL: url)
        
        // Usando el metodo POST:
        req.HTTPMethod = "POST"
        
        // La cabecera Authorization:
        let bearer_token = "\(CONSUMER_KEY):\(CONSUMER_SECRET)".stringByAddingPercentEscapesUsingEncoding(NSUTF8StringEncoding)!
        let base64_bearer_token = bearer_token.dataUsingEncoding(NSUTF8StringEncoding, allowLossyConversion: false)?.base64EncodedStringWithOptions(.allZeros)
        req.addValue("Basic \(base64_bearer_token!)", forHTTPHeaderField: "Authorization")
        
        // La cabecera Content-Type:
        req.addValue("application/x-www-form-urlencoded;charset=UTF-8", forHTTPHeaderField: "Content-Type")
        
        // El Body:
        let body = "grant_type=client_credentials"
        let dbody = body.dataUsingEncoding(NSUTF8StringEncoding, allowLossyConversion: false)
        
        // Crear una sesion
        let session = NSURLSession(configuration: NSURLSessionConfiguration.defaultSessionConfiguration())
        
        // Crear una tarea para subir la peticion y bajar el token de acceso:
        let task = session.uploadTaskWithRequest(req, fromData: dbody) { (data: NSData!, res: NSURLResponse!, error: NSError!) in
            
            if error == nil && (res as! NSHTTPURLResponse).statusCode == 200 {
                var err: NSError?
                if let dic = NSJSONSerialization.JSONObjectWithData(data, options: .allZeros, error: &err) as? [String:String] {
                    
                    println("Bien, ya tengo el token de acceso 😃😃😃")
                    
                    Static.access_token = dic["access_token"]
                }
            }
            
            completion(token: Static.access_token)
        }
        task.resume()
    }
}

